#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#if defined __BORLANDC__
	#include "icedos.h"
#else
	#include "icelinux.h"
#endif
#include "ice16f84.h"
#include "icelpt.h"
#include "icescrn.h"
#include "icerout.h"

int main(int argc, char *argv[])
	{
	int i, retaddr=0xFFFF, runmode=1, teller=0;
	char *text,*text1;
	char c;
	FILE *hexfile;

	if (argc!=2)
		{
		printf("ICE16F84 - In-Circuit Emulator for PIC 16F84\n"
		       "Type ice --help for instructions\n\n");
		exit(1);
		}
	if (strcmp(argv[1],"--help")==0)
		{
		printf("ICE16F84 - In-Circuit Emulator for PIC 16F84\n"  
		       "Usage: ice <hexfile>    where <hexfile> is the programfile in INHX8M format\n"  
		       "       ice --help       displays this helptext\n"
		       "       ice --version    displays the version and build number\n\n"  
		       "ICE16F84 is GPL-software\n\n");
		exit(1);
		}
	if (strcmp(argv[1],"--version")==0)
		{
		printf("ICE16F84 - In-Circuit Emulator for PIC 16F84\n");
		printf("%s for %s, (c) Roland Leurs\n\n", version, platform);  
		exit(1);
		}

	if(strcmp(argv[1],"--iospeed")==0)
		{
		long i,n=2500000;
		long start,end;
		float utime,ops;
		progstart();
		initport();
		printf("Performing %ld I/O read and write operations\n", n);
		start=time(0);
		i=n;
		while (i-- > 0)
			outb(inb(BASEPORT+1), BASEPORT);
		end=time(0);
		ops=2*n/(end-start);
		utime=1000000/ops;
		printf("This took %ld seconds, i.e. %1.0f operations per second\n", end-start, ops);
		printf("or %1.2f microseconds per operation\n", utime);
		progend();
		exit(1);
		}

	hexfile = fopen(argv[1],"r");
	if (!hexfile)
		{
		printf("ICE16F84 couldn't open %s as program file\n", argv[1]);
		perror("Reason");
		exit(2);
		}
	
	text=malloc(80);
	text1=malloc(80);
	if (text == NULL || text1 == NULL)
		{
		perror("Allocation for `text' failed");
		exit(3);
		}
	while((c=getc(hexfile)) != EOF)
		{
		unsigned int n, adres, j, value;
		i=0;
		if(c==0x0D) c=getc(hexfile);
		if(c!=':' && c!=-1)
			{
			printf("Not a valid INHX8M file format!\n");
			exit(4);
			}
		while(((c=getc(hexfile)) != EOF) && c!=0x0A)
			{
			text[i++]=toupper(c);
			text[i]=0x00;
			}
		text1[0]=text[0];
		text1[1]=text[1];
		text1[2]=0x00;
		n=xtoi(text1)/2;
		for(i=2; i<6; i++)
			{
			text1[i-2]=text[i];
			text1[i-1]=0x00;
			}
		adres=xtoi(text1)/2;
		teller=8;
		for(i=0; i<n; i++)
			{
			for(j=0; j<4; j++)
				{
				text1[j]=text[teller++];
				text1[j+1]=0x00;
				}
			value=((xtoi(text1)&0xFF)*256)+((xtoi(text1)&0xFF00)>>8);
			if(adres==0x2007)	/* set configuration word */
				config=value;
			else if (adres>0x20FF && adres<0x2180)
				eeprom[adres & 0x3F]=value;
			else
				progmem[adres & 0x03FF]=value;
			adres++;
			}
		}
	fclose(hexfile);

	for(i=0; i<8; i++)
		breakpoint[i]=0xFFFF;

	progstart();
	initport();
	buildscreen(argv[1]);
	pic_power_on();
	showkeyinfo();
	showregs();
	showdisas(0x0000);
	update_view_window();
	c=readkey();
	while (toupper(c)!=key_f2)
		{
		if (toupper(c) == key_pgdn) showdisas(first_disas_address+11);
		if (toupper(c) == key_pgup) showdisas(first_disas_address-11);
		if (toupper(c) == key_curdn) showdisas(first_disas_address+1);
		if (toupper(c) == key_curup) showdisas(first_disas_address-1);
		if (toupper(c) == key_h) showhelp();
		if (toupper(c) == key_s) breakpointmenu();
		if (toupper(c) == key_g) runmode=0;
		if (c == key_1) showfile();
		if (c == key_2) showstack();
		if (c == key_3) showeeprom();
		if (c == key_9) showconfig();
		if (c == key_0) showbreakpoints();
		if (c == space)	/* step */
			{
			if(iscall(pc))
				{
				retaddr=pc+1;
				runmode=0;
				}
			else
				c=enter;
			}
		if (c == enter) /* trace */
			{
			execinstr(pc);
			showdisas(pc);
			regfile[PCL]=pc&0xFF;
			showregs();
			update_view_window();
			}
		if (toupper(c) == key_r)
			{
			pic_reset(1);
			showdisas(first_disas_address);
			showregs();
			update_view_window();
			}
		if (runmode==1)
			c=readkey();
		while(runmode!=1)
			{
			if(runmode==0) runmode=runmsg();
			execinstr(pc);
			gotoxy(50,21);
			cprintf("%0.4X\n", pc);
			if (kbhit())
				c=readkey();
			else
				c=0;
			for(i=0; i<9; i++)
				{
				if((pc==breakpoint[i] && i<8) || ((pc==retaddr || c==space || c==enter) && i==8))
					{
					runmode=1;
					showdisas(pc);
					showregs();
					update_view_window();
					if (i==8)
						retaddr=0xFFFF;
					c=readkey();
					}
				}
			}
		}
	progend();
	free(text);
	free(text1);
	clrscr();
	return(0);
	}
